import os, sys, fnmatch, optparse, subprocess, shutil

def manageArgs():

   parser = optparse.OptionParser(usage="Usage: %prog [options]",  version="%prog 1.0")
   # Gestion de la configuration pour la compilation
   parser.add_option("--build",
                     action="store",
                     type="choice",
                     choices=["release", "debug", "doc"],
                     dest="buildArgs",
                     default="release",
                     help='Compile les sources selon le mode utilise. [release (defaut)|debug|doc]')

   # Selectionne la cible
   parser.add_option("--output",
                     action="store",
                     type="string",
                     dest="outputArgs",
                     default="cdtheque",
                     help='Defini le nom de la cible a compiler (cdtheque par defaut)')

   # Selectionne le chemin d'installation de DMD
   parser.add_option("--dmd",
                     action="store",
                     type="string",
                     dest="dmdPathArgs",
                     default="",
                     help='Defini le chemin du compilateur DMD')


   (options, args) = parser.parse_args()

   return options


def main():
   options = manageArgs()

   build   = options.buildArgs
   out     = os.path.join(build,options.outputArgs)
   dmd     = options.dmdPathArgs
   srcList = []

   for item in os.listdir('.'):
      if fnmatch.fnmatch(item, "*.d"):
         srcList.append(item)


   if os.path.exists(build):
      shutil.rmtree(build, ignore_errors=True)
      os.makedirs(build)

   if build == "release":
      args = [os.path.join(dmd,'dmd'), '-of'+out, '-release', '-L-ldl', '-L-lxml2']
      args.extend(srcList)
   elif build == "debug":
      args = [os.path.join(dmd,'dmd'), '-of'+out, '-debug', '-g', '-L-ldl', '-L-lxml2']
      args.extend(srcList)
   else:
      args = [os.path.join(dmd,'dmd'), '-c', '-D', '-Dddoc']
      args.extend(srcList)

   pipe = subprocess.Popen(args)
   pipe.wait()

   if pipe.returncode != 0:
      print("Le build '"+build+"'' a echoue !")
   else:
      print("Compilation de '"+build+"'' terminee.")

if __name__ == "__main__":
   main()