import std.stdio;

/++
  La classe ElementCD est la classe de manipulation des données concernant un Album CD.
  Ci-dessous la liste des méthodes disponibles.
  Exemple:
  ---
   import std.stdio;
   import std.conv;

   import elements;

   void main(string[] args)
   {
      ElementCD item = new ElementCD;
      item.albumName = "Album 1";
      item.trackName = "Track 1";
      item.trackName = "Track 2";
      item.trackName = "Track 3";
      item.trackName = "Track 4";
      item.trackName = "Track 5";
      item.albumTime = 33;

      writeln("Album : ", item.albumName);
      writeln("Liste des pistes :");
      foreach(curItem ; item.trackName)
      {
         writeln(curItem);
      }
      writeln("Durée du CD : ", to!string(item.albumTime));
      item.replaceTrackName(4, "Track 0");
      writeln("Liste des pistes après remplacement de l'élément 4:");
      foreach(curItem ; item.trackName)
      {
         writeln(curItem);
      }
      item.removeTrackName(2);
      writeln("Liste des pistes après suppression de l'élément 2:");
      foreach(curItem ; item.trackName)
      {
         writeln(curItem);
      }
   }
  ---
+/

class ElementCD
{
   private :
      static MAX_TIME = 75; /* Durée max du CD */
      string _albumName;    /* Nom de l'album. */
      string[] _tracksName; /* Tableau contenant la liste de toutes les chansons. */
      uint   _totalTime;    /* Durée de l'album en minutes. */

   public:
      /* Construteur */
      this()
      {
         _albumName = "";
         _totalTime = 0;
      }

      /** Récupère le nom de l'album.*/
      @property string albumName() {return _albumName;}
      /** Défini le nom de l'album avec albumName.*/
      @property void albumName(string albumName) {_albumName = albumName;}
      /** Retourne un tableau de string contenant la liste des chansons. */
      @property string[] trackName() {return _tracksName;}
      /** Retourne la string de lélément pointé par item. */
      @property string trackName(uint item) {return item < _tracksName.length ? _tracksName[item] : "";}
      /** Ajoute une nouvelle piste trackName. */
      @property void trackName(string trackName)
      {
         if(trackName.length > 0)
         {
            _tracksName ~= trackName;
         }
      }
      /** Remplace la string, dans l'élément indiqué pointé par indexName, par trackName. */
      void replaceTrackName(uint indexName, string trackName)
      {
         if(trackName.length > 0 && indexName < _tracksName.length)
         {
            _tracksName[indexName] = trackName;
         }
      }
      /** Supprime l'élément pointé par indexName. */
      void removeTrackName(uint indexName)
      {
         /* premier élément à supprimer */
         if(indexName == 0)
         {
            _tracksName = _tracksName[1.._tracksName.length];
         }
         else
         /* dernier élément à supprimer */
         if(indexName == _tracksName.length-1)
         {
            _tracksName = _tracksName[0.._tracksName.length-1];
         }
         else
         /* Sinon un élément dans la liste */
         if(indexName < _tracksName.length)
         {
            _tracksName = _tracksName[0..indexName] ~ _tracksName[indexName+1.._tracksName.length];
         }
      }
      /** Récupère la durée totale de l'album. */
      @property uint albumTime() {return _totalTime;}
      /** Défini la durée de l'album par timeAlbum et vérifie qu'elle ne dépasse pas la capacité du CD (fixé ici à 74 minutes).*/
      @property void albumTime(uint timeAlbum)
      {
         if(timeAlbum > 0 && timeAlbum < MAX_TIME)
         {
            _totalTime = timeAlbum;
         }
      }
}