import std.xml;
import std.stdio;
import std.string;
import std.conv;
import std.file;

import elements;

/**
*  Écrit le contenu du catalogue contenu en mémoire dans un fichier XML. Si le fichier existe déjà il sera écrasé.
*
*  Paramètres:
*
*  fileName     = Le nom du fichier XML qui sera créé.
*
*  listElements = Le catalogue stocké en mémoire.
**/
void xmlWrite(string fileName, ElementCD[] listElements)
{
   if(fileName.length > 0 && listElements.length > 0)
   {
      auto doc = new Document(new Tag("catalogue"));
      /* Parcours tous les catalogues */
      foreach(album ; listElements)
      {
         if(album !is null)
         {
            auto element = new Element("album");
            element.tag.attr["nom"] = album.albumName;
            foreach(allItem ; album.trackName)
            {
               if(allItem !is null)
               {
                  element ~= new Element("titre", allItem);
               }
            }
            element ~= new Element("duree", to!string(album.albumTime));
            /* Génère l'arbre XML */
            doc ~= element;
         }
      }
      try
      {
         std.file.write(fileName, join(doc.pretty(3),"\n"));
      }
      catch(FileException ex)
      {
         writeln(ex.msg);
         writeln("Pressez Entrée pour retourner au menu.");
         readln();
      }
   }
   else
   {
      writeln("Aucun nom de fichier ou rien à enregistrer !");
   }
}

/**
*  Récupère le contenu d'un catalogue à partir d'un fichier XML.
*
*  Paramètre:
*
*  fileName = Le nom du fichier XML qui sera lu.
*
*  Retour:
*
*  Retourne un tableau de la classe ElementCD si lecture valide ; sinon retourne un tableau vide.
**/
ElementCD[] xmlRead(string fileName)
{
   ElementCD[] allAlbum;

   if(fileName.length > 0)
   {
      string xmlData;
      /* Lecture du catalogue XML */
      try
      {
         xmlData = cast(string)std.file.read(fileName);
      }
      catch(FileException ex)
      {
         writeln(ex.msg);
         writeln("Pressez Entrée pour retourner au menu.");
         readln();
         return allAlbum;
      }
      /* Valide le XML au cas ou il serait corrompu */
      try
      {
         check(xmlData);
      }
      catch(CheckException ex)
      {
         writeln(ex.msg);
         writeln("Pressez Entrée pour retourner au menu.");
         readln();
         return allAlbum;
      }
      /* Création d'un arbre XML à partir des données du fichier */
      auto xmlDoc = new DocumentParser(xmlData);

      /* Récupère les albums et les ajoute au tableau de la classe ElementCD */
      xmlDoc.onStartTag["album"] = (ElementParser xmlDoc)
      {
         ElementCD album = new ElementCD;
         album.albumName = xmlDoc.tag.attr["nom"];

         xmlDoc.onEndTag["titre"] = (in Element e) { album.trackName = e.text(); };
         xmlDoc.onEndTag["duree"] = (in Element e) { album.albumTime = to!uint(e.text()); };

         xmlDoc.parse();

         allAlbum ~= album;
      };
      xmlDoc.parse();
   }
   else
   {
      writeln("Aucun nom de fichier !");
   }
   return allAlbum;
}